public class BankAccountTriggerHandler {
    public static void handleUpdatedAccounts(Map<Id, Account__c> newMap, Map<Id, Account__c> oldMap) {
        checkUpdatedSyncDate(newMap, oldMap);
    }
    
    private static void checkUpdatedSyncDate(Map<Id, Account__c> newMap, Map<Id, Account__c> oldMap) {
        List<Account__c> toSync = new List<Account__c>();
        Map<Id, Account__c> shorterSyncMap = new Map<Id, Account__c>();
        for (Account__c a: newMap.values()) {
            if (a.Sync_From__c != oldMap.get(a.Id).Sync_From__c) {
                toSync.add(a);
                
                if (a.Sync_From__c > oldMap.get(a.Id).Sync_From__c) {
                    shorterSyncMap.put(a.Id, a);
                } 
            }
        }
        
        if (!toSync.isEmpty()) {
            System.enqueueJob(new QueueableAccountSync(toSync));
        }
        
        if (!shorterSyncMap.isEmpty()) {
            deleteOlderTransactions(shorterSyncMap);
        }
    }
    
    private static void deleteOlderTransactions(Map<Id, Account__c> accountsMap) {
        List<Transaction__c> allTransactions = TransactionSelector.getTransactionsByAccountIds(accountsMap.keySet());
        List<Transaction__c> toDelete = new List<Transaction__c>();    
        
        for (Transaction__c t: allTransactions) {
            if (t.Date__c < accountsMap.get(t.Account__c).Sync_From__c) {
                toDelete.add(t);
            }
        }        
        
        delete toDelete;
    }
}