public class FioConnector {
    
    public class Statement {
        public Info accountStatement;
    }
    
    public class Info {
        public BankAccount info;
        public TransactionList transactionList;
    }
    
    public class TransactionList {
        public List<TransactionX> transaction_x;
    }
    
    public class BankAccount {
        public String accountId;
        public String bankId;
        public String currency_x;
        public String iban;
        public String bic;
        public Decimal openingBalance;
        public Decimal closingBalance;
        public String dateStart;
        public String dateEnd;
        public String yearList;
        public Integer idList;
        public Long idFrom;
        public Long idTo;
        public Integer idLastDownload;
        
        public Account__c toSobject() {
            return new Account__c(
                Name = this.accountId + '/' + this.bankId,
                Account_Number__c = this.accountId,
                Bank_Code__c = this.bankId,
                Currency__c = this.currency_x,
                IBAN__c = this.iban,
                BIC__c = this.bic,
                Balance__c = this.closingBalance,
                Last_Sync__c = System.now(),
                Last_Sync_Status__c = Constants.successStatus
            );
        }
    }
    
    public class StringValue {
        public String value;
        public String name;
        public String id;
    }
    
    public class DecimalValue {
        public Decimal value;
        public String name;
        public String id;
    }
    
    public class IntegerValue {
        public Integer value;
        public String name;
        public String id;
    }
    
    public class LongValue {
        public Long value;
        public String name;
        public String id;
    }
    
    public class DateValue {
        public String value;
        public String name;
        public String id;
        
        public String convert(String val) {
            String dateRegEx = '^[0-9]{4}[- /.][0-9]{1,2}[- /.][0-9]{1,2}.*';
            Pattern myPattern = Pattern.compile(dateRegEx);
            Matcher myMatcher = myPattern.matcher(val);
            
            string y;
            string m;
            string d;
            
            if (myMatcher.matches()) {
                string valu;
                date datum;
                valu = val.substring(0, 10);
                datum = date.valueOf(valu);
                y = string.valueOf(datum.year());
                
                if (datum.month() < 10) {
                    m = '0' + datum.month();
                } else {
                    m = '' + datum.month();
                }
                
                if (datum.day() < 10) {
                    d = '0' + datum.day();
                } else {
                    d = '' + datum.day();
                }
            } else {
                String dateRegEx2 = '^[0-9]{8,20}';
                Pattern myPattern2 = Pattern.compile(dateRegEx2);
                Matcher myMatcher2 = myPattern2.matcher(val);
                
                if (myMatcher2.matches()) {
                    Long milisec = Long.valueOf(val);
                    DateTime dateInstance = datetime.newInstanceGmt(1970, 1, 1, 0, 0, 0);
                    String dateTimeValue = dateInstance.addSeconds(integer.valueOf(milisec)).format();
                    String dateShort = dateTimeValue.substring(0, 10);
                    DateValue dateConv = new DateValue();
                    date dpar = date.parse(dateShort);
                    
                    y = string.valueOf(dpar.year());
                    
                    if (dpar.month() < 10) {
                        m = '0' + dpar.month();
                    } else {
                        m = '' + dpar.month();
                    }
                    
                    if (dpar.day() < 10) {
                        d = '0' + dpar.day();
                    } else {
                        d = '' + dpar.day();
                    }
                }
            }
            
            return y + '-' + m + '-' + d;
        }
    }
    
    public class TransactionX {
        public DateValue column0; // Datum
        public DecimalValue column1; // Objem
        public StringValue column2; // Protiúčet
        public StringValue column3; // Kód banky
        public StringValue column4; // KS
        public StringValue column5; // VS
        public StringValue column6; // SS
        public StringValue column7; // Uživatelská identifikace
        public StringValue column8; // Typ
        public StringValue column9; // Provedl
        public StringValue column10; // Název protiúčtu
        public StringValue column12; // Název banky
        public StringValue column14; // Měna
        public StringValue column16; //Upřesnění či Zpráva pro příjemce či ...
        public LongValue column17; // ID pokynu
        public LongValue column22; // ID pohybu
        public StringValue column25; // Komentář
        public StringValue column26; // BIC
        
        public Transaction__c toSObject() {
            Decimal amount = column1 == null ? null : column1.value;
            
            string c = column0.convert(column0.value);

            return new Transaction__c(
                External_ID__c = '' + column22.value,
                Date__c = column0 == null ? null : Date.valueOf(c),
                Amount__c = amount,
                Bank_Account__c = column2 == null ? '' : column2.value, // Protiúčet
                Bank_Account_Code__c = column3 == null ? '' : column3.value, // Kod banky
                BIC__c = column26 == null ? '' : column26.value, // Kód banky
                KS__c = column4 == null ? '' : column4.value, // KS
                VS__c = column5 == null ? '' : column5.value, // VS
                SS__c = column6 == null ? '' : column6.value, // SS
                User_Identification__c = column7 == null ? '' : column7.value, // Uživatelská identifikace
                Type__c = column8 == null ? '' : column8.value, // Typ
                Created_By__c = column9 == null ? '' : column9.value, // Provedl
                Bank_Account_Name__c  = column10 == null ? '' : column10.value, // Název protiúčtu
                Bank_Name__c = column12 == null ? '' : column12.value, // Název banky
                Command_Id__c = column17 == null ? null : string.valueOf(column17.value), // ID pokynu
                Comment__c = column25 == null ? '' : column25.value,//column25.value.contains('dne') ? column25.value.substringBefore(', dne') : column25.value, // Komentář
                Message__c  = column16 == null ? '' : column16.value
            );
        }
        
    }
    
    public static Statement readData(String token, Date syncFrom, Date syncTo) {
        HttpRequest req = new HttpRequest();
        String endpoint = 'https://www.fio.cz/ib_api/rest/periods/' + token + 
            + '/' + formatDate(syncFrom == null ? System.today() : syncFrom) + '/' + formatDate(syncTo == null ? System.today() : syncTo) + '/transactions.json';
        
        req.setEndpoint(endpoint);
        req.setMethod('GET');
        Http h = new Http();
        HttpResponse res = h.send(req);
        
        String s = res.getBody().replace('"transaction":', '"transaction_x":');
        
        Statement st = (Statement) JSON.deserialize(s, Statement.class);

        return st;
    }
    
    public static List<Transaction__c> getTransactions(String accountId, Statement st) {
        List<Transaction__c> out= new List<Transaction__c>();
        if (st.accountStatement != null && st.accountStatement.transactionList != null && st.accountStatement.transactionList.transaction_x != null) {
            for (TransactionX t: st.accountStatement.transactionList.transaction_x) {
                Transaction__c tr = t.toSObject();
                tr.Account__c = accountId;
                tr.External_Id__c = tr.External_Id__c + accountId;
                
                out.add(tr);
            }
        }
        
        return out;
    }
    
    private static String formatDate(Date toFormate) {
        return toFormate.year() + '-' + toFormate.month() + '-' + toFormate.day();
    }
    
}