public class QueueableAccountSync implements System.Queueable, Database.AllowsCallouts {
    List<Account__c> toSync = new List<Account__c>();
    
    public QueueableAccountSync(List<Account__c> toSync) {
        this.toSync = toSync;
    }
    
    public void execute(QueueableContext context) {
        syncAccount(toSync);
    }
    
    public static void syncAccount(List<Account__c> toSync) {
        while(toSync.size() > 0) {
            if (Limits.getDMLStatements() < Limits.getLimitDMLStatements() && !Test.isRunningTest()) {
				Account__c acc = toSync.remove(0);
                
                FioConnector.Statement st = FioConnector.readData(
                        BankAccountSelector.getSettingByAccountNumber(
                            acc.Account_Number__c + '/' + acc.Bank_Code__c).Api_Token__c, 
                        	Date.newInstance(acc.Sync_From__c.year(), acc.Sync_From__c.month(), acc.Sync_From__c.day()), null);
                
                acc.Balance__c = st.accountStatement.info.closingBalance;
                acc.Last_Sync__c = System.now();
                acc.Last_Sync_Status__c = Constants.successStatus;
                
                update acc;
                
                upsert FioConnector.getTransactions(
                    acc.Id, 
                    st
                ) External_Id__c;
			} else {
                break;
            }
        }
        
        if (toSync.size() > 0 && !Test.isRunningTest()) {
            QueueableAccountSync q = new QueueableAccountSync(toSync);
            
            System.enqueueJob(q);
        }
    }
    
}