public class TransactionTriggerHandler {
    public static void checkRelatedInvoices(List<Transaction__c> transactions) {
        Set<Id> accountIds = new Set<Id>();
        List<Transaction__c> tWithVs = new List<Transaction__c>();
        
        for (Transaction__c t: transactions) {
            if (t.Vs__c != null && t.Type__c.contains('Příjem')) {
                tWithVs.add(t);
                accountIds.add(t.Account__c);
            }
        }
        
        if (tWithVs.isEmpty()) {
            return;
        }
        
        Map<Id, Map<String, String>> accountToInvoiceMap = InvoiceSelector.getAccountToInvoiceIds(accountIds);

        for (Transaction__c t: tWithVs) {
            if (!accountToInvoiceMap.containsKey(t.Account__c)) {
                continue;
            }
            
            if (accountToInvoiceMap.get(t.Account__c).containsKey(t.Vs__c)) {
                t.Invoice__c = accountToInvoiceMap.get(t.Account__c).get(t.Vs__c);
            }
        }
    }
    
    public static void processIncomeTransactions(List<Transaction__c> transactions) {
        List<Transaction__c> incomeTransactions = new List<Transaction__c>();
        
        for (Transaction__c t: transactions) {
            if (t.Type__c.contains('Příjem') && t.Vs__c != null) {
                incomeTransactions.add(t);
            }
        }
        
        if (incomeTransactions.isEmpty()) {
            return;
        }
        
        updateInvoices(incomeTransactions);
    }
    
    private static void updateInvoices(List<Transaction__c> transactions) {
        Set<Id> accountIds = new Set<Id>();
        for (Transaction__c t: transactions) {
            accountIds.add(t.Account__c);
        }
        
        Map<Id, Map<String, Invoice__c>> AccountToInvoiceMap = InvoiceSelector.getAccountToUnpaidInvoices(accountIds);
        if (AccountToInvoiceMap.isEmpty()) {
            return;
        }
        
        List<Messaging.SingleEmailMessage> emails = new List<Messaging.SingleEmailMessage>();
        List<Invoice__c> toUpdate = new List<Invoice__c>();
        for (Transaction__c t: transactions) {
            if (!AccountToInvoiceMap.containsKey(t.Account__c)) {
                continue;
            } else if (!AccountToInvoiceMap.get(t.Account__c).containsKey(t.Vs__c)) {
                continue;
            }
            
            Invoice__c invoice = AccountToInvoiceMap.get(t.Account__c).get(t.Vs__c);
            invoice.Paid_Amount__c += t.Amount__c;
            
            if (invoice.Paid_Amount__c >= invoice.Amount__c) {
                invoice.Paid__c = true;
                toUpdate.add(invoice);
                emails.add(buildEmail(invoice));
            }
        }
        
        if (!emails.isEmpty()) {
            try {
                Messaging.sendEmail(emails,false);
                update toUpdate;
            } catch (Exception e) {
                System.debug(e.getMessage());
            }
        }
    }
    
    private static Messaging.SingleEmailMessage buildEmail(Invoice__c invoice) {
        Messaging.SingleEmailMessage out = new Messaging.SingleEmailMessage();
        
        String body = 'Dobrý den, \n';
        body += 'děkujeme vám za zaplacení celé dlužné částky ve výši: ' + invoice.Amount__c;
        body += ' za fakturu: ' + invoice.Name;
        
        out.setToAddresses(new List<String>{invoice.Email__c});
        out.setPlainTextBody(body);
        out.setSubject('Děkujeme vám za zaplacení');
        out.setCharset('UTF-8');
        
        return out;
    }
}